"""
@author: IOB

Apply UMAP dimensions reduction to 2D and plot.
"""

import matplotlib.pyplot as plt
import pandas as pd
from sklearn.preprocessing import StandardScaler, LabelEncoder
import umap

plt.style.use('seaborn-white')
plt.rcParams.update({'font.size': 16})
plt.rcParams["font.family"] = "Arial"

# Data prep

df = pd.read_csv('./data/CannaLit.csv').dropna()
y, x = df.loc[:, 'yield'],df.iloc[:, 1:-2] 

X_train = StandardScaler().fit_transform(x)
strain = LabelEncoder().fit_transform(df['strain'])

## UMAP for Visualisation ##

reducer_vis = umap.UMAP(
    random_state=25,
    min_dist=0.5,  # Default is 0.1
    n_neighbors= 50,  # Default is 15
    n_components=2)

umap_2d = reducer_vis.fit_transform(X_train)

data = pd.DataFrame(umap_2d,df['strain'].values)

for strain in df['strain'].unique(): 
    plt_data = data.loc[strain,:]
    if strain == 'Lifter':
        plt.scatter(plt_data.iloc[:,0],plt_data.iloc[:,1], label='DoE Data', c='#4c72b0')
    elif strain == 'Indica Princess':
        plt.scatter(plt_data.iloc[:,0],plt_data.iloc[:,1], label='Out of Sample', c='#c44f51')
    else:
        plt.scatter(plt_data.iloc[:,0],plt_data.iloc[:,1], c='#8c8c8c')

plt.xlabel('UMAP 1')
plt.ylabel('UMAP 2')
plt.legend()
# plt.savefig('./figure.svg', bbox_inches='tight')
plt.show()
